@@ -659,6 +659,10 @@ API_EXPORTED usb_dev_handle *usb_open(struct usb_device *dev)
 
 	r = libusb_open((libusb_device *) dev->dev, &udev->handle);
 	if (r < 0) {
+		if (r == LIBUSB_ERROR_ACCESS) {
+			usbi_info("Device open failed due to a permission denied error.");
+			usbi_info("libusb requires write access to USB device nodes.");
+		}
 		usbi_err("could not open device, error %d", r);
 		free(udev);
 		errno = libusb_to_errno(r);
@@ -773,7 +777,7 @@ API_EXPORTED int usb_bulk_read(usb_dev_handle *dev, int ep, char *bytes,
 	return usb_bulk_io(dev, ep, bytes, size, timeout);
 }
 
-API_EXPORTED int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes,
+API_EXPORTED int usb_bulk_write(usb_dev_handle *dev, int ep, const char *bytes,
 	int size, int timeout)
 {
 	if (ep & USB_ENDPOINT_IN) {
@@ -784,7 +788,7 @@ API_EXPORTED int usb_bulk_write(usb_dev_handle *dev, int ep, char *bytes,
 		ep &= ~USB_ENDPOINT_IN;
 	}
 
-	return usb_bulk_io(dev, ep, bytes, size, timeout);
+	return usb_bulk_io(dev, ep, (char *)bytes, size, timeout);
 }
 
 static int usb_interrupt_io(usb_dev_handle *dev, int ep, char *bytes,
@@ -817,7 +821,7 @@ API_EXPORTED int usb_interrupt_read(usb_dev_handle *dev, int ep, char *bytes,
 	return usb_interrupt_io(dev, ep, bytes, size, timeout);
 }
 
-API_EXPORTED int usb_interrupt_write(usb_dev_handle *dev, int ep, char *bytes,
+API_EXPORTED int usb_interrupt_write(usb_dev_handle *dev, int ep, const char *bytes,
 	int size, int timeout)
 {
 	if (ep & USB_ENDPOINT_IN) {
@@ -828,7 +832,7 @@ API_EXPORTED int usb_interrupt_write(usb_dev_handle *dev, int ep, char *bytes,
 		ep &= ~USB_ENDPOINT_IN;
 	}
 
-	return usb_interrupt_io(dev, ep, bytes, size, timeout);
+	return usb_interrupt_io(dev, ep, (char *)bytes, size, timeout);
 }
 
 API_EXPORTED int usb_control_msg(usb_dev_handle *dev, int bmRequestType,
@@ -912,6 +916,22 @@ API_EXPORTED int usb_get_driver_np(usb_dev_handle *dev, int interface,
 
 API_EXPORTED int usb_detach_kernel_driver_np(usb_dev_handle *dev, int interface)
 {
-	return compat_err(libusb_detach_kernel_driver(dev->handle, interface));
+	int r = compat_err(libusb_detach_kernel_driver(dev->handle, interface));
+	switch (r) {
+	case LIBUSB_SUCCESS:
+		return 0;
+	case LIBUSB_ERROR_NOT_FOUND:
+		return -ENODATA;
+	case LIBUSB_ERROR_INVALID_PARAM:
+		return -EINVAL;
+	case LIBUSB_ERROR_NO_DEVICE:
+		return -ENODEV;
+	case LIBUSB_ERROR_OTHER:
+		return -errno;
+	/* default can be reached only in non-Linux implementations,
+	 * mostly with LIBUSB_ERROR_NOT_SUPPORTED. */
+	default:
+		return -ENOSYS;
+	}
 }
 
